from enum import Enum


class TicktickTaskParameters(Enum):
    ID = "id"
    PROJECT_ID = "projectId"
    SORTORDER = "sortOrder"
    TITLE = "title"
    CONTENT = "content"
    DESC = "desc"
    START_DATE = "startDate"
    DUE_DATE = "dueDate"
    TIMEZONE = "timeZone"
    IS_FLOATING = "isFloating"
    IS_ALL_DAY = "isAllDay"
    REMINDERS = "reminders"
    REPEAT_FIRST_DATE = "repeatFirstDate"
    REPEAT_FLAG = "repeatFlag"
    EX_DATE = "exDate"
    COMPLETED_TIME = "completedTime"
    COMPLETED_USER_ID = "completedUserId"
    REPEAT_TASK_ID = "repeatTaskId"
    PRIORITY = "priority"
    STATUS = "status"
    ITEMS = "items"
    PROGRESS = "progress"
    MODIFIED_TIME = "modifiedTime"
    ETAG = "etag"
    DELETED = "deleted"
    CREATED_TIME = "createdTime"
    CREATOR = "creator"
    TAGS = "tags"
    REPEAT_FROM = "repeatFrom"
    POMODORO_SUMMARIES = "pomodoroSummaries"
    FOCUS_SUMMARIES = "focusSummaries"
    FOCUS_TIME = "focus_time"
    POMO_DURATION = "pomoDuration"
    STOPWATCH_DURATION = "stopwatchDuration"
    COLUMN_ID = "columnId"
    PARENT_ID = "parentId"
    KIND = "kind"
